import { importRouters } from "../lib.js";
import multer from "multer";
import cookieParser from "cookie-parser";
import { initDb } from "../db.js";
import express from "express";

let db = await initDb();

const upload = multer({ dest: 'uploads/' });

const aliases = {
    '/': '/walls/get/hub/main/0'
};

const routers = {
    '/you': './routes/you.js',
    '/api/form/you': './routes/youApi.js',
    '/api/file/comment': './routes/commenter.js',
    '/walls': './routes/walls.js',
    '/comment': './routes/comment.js'
}

function doAliases(app) {
    for (let alias in aliases) {
        let aliasV = aliases[alias];
        app.all(alias, (req, res, next) => {
            res.redirect(aliasV)
        })
    }

}

async function auther(req, res, next) {
    let { token } = req.cookies;

    if (!token) {
        res.auth = {};
        next();
        return;
    }

    let match = await db.all('SELECT * FROM token WHERE token = ?', [
        token || 'blah'
    ]);

    let username = match[0] ? match[0].username : '!nobody';

    let valid = false;

    if (username != '!nobody') {
        valid = await db.all('SELECT * FROM auth WHERE username = ?', [
            username
        ]);
    }

    res.auth = {
        username,
        valid: valid[0]
    };

    next();
}

function initr(req, res, next) {
    let headerCtx = [
        { link: '/walls/get/home/0', icon: '/icon.svg', name: 'DervNet' },
        { link: '/walls/list', icon: '/walls.svg', name: 'Explore' },
        { link: '/you/logout', icon: '/logout.svg', name: 'Leave' } // fix icon
    ];

    if (!res.auth || !res.auth.valid) {
        headerCtx = [
            { link: '/walls/get/home/0', icon: '/icon.svg', name: 'DervNet' },
            { link: '/walls/list', icon: '/walls.svg', name: 'Explore' },
            { link: '/you/login', icon: '/login.svg', name: 'Log in' },
            { link: '/you/new', icon: '/join.svg', name: 'Join' }
        ];
    }

    res.ctx = {
        mainPage: '404.ejs',
        mainCtx: {},
        headerCtx
    };
    next();
}

async function doInit(app) {
    app.use(cookieParser());

    doAliases(app);

    app.use(express.static('public'));
    app.use('/uploads', express.static('uploads'));
    app.use(auther)
    app.use(initr)

    app.use('/api/form', upload.none());
    app.use('/api/file', upload.single('file'));

    await importRouters(app, routers);

    app.use('/api', (req, res, next) => {
        res.send(res.api);
    })

    app.use((req, res, next) => {
        res.render('root.ejs', res.ctx)
    })
}

export { doInit };